#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define ABS(x) ((x) > 0 ? (x) : (-(x)))



#define S1_CHN 0
#define S2_CHN 1
#define S3_CHN 2
#define S4_CHN 3
#define S5_CHN 6



int sensor[5];
int threshold[5] = {640, 680, 320, 640, 360};
int chan[5] = {0, 1, 2, 3, 6};


void init_adc()
{
	ADMUX |= (1 << REFS0) | (1 << REFS1);
	ADCSRA |= (1 << ADEN) | (1 << ADIE) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
}

void init_timers()
{
	TCCR0 = (1 << WGM00) | (1 << WGM01) | (1 << COM01) | (1 << CS02) | (1 << CS00);
	TCCR2 = (1 << WGM20) | (1 << WGM21) | (1 << COM21) | (1 << CS22) | (1 << CS21) | (1 << CS20);
	DDRB |= (1 << PB3);
	DDRD |= (1 << PD7);
}

//--------------------------------


void inline left_motor_forward(int speed)
{
	PORTC &= ~((1 << PC4) | (1 << PC5));
	PORTC |= (1 << PC5);
	
	
	OCR2 = speed;
}


void inline right_motor_forward(int speed)
{
	PORTC &= ~((1 << PC6) | (1 << PC7));
	PORTC |= (1 << PC7);

	OCR0 = speed;
}

void inline left_motor_backward(int speed)
{
	PORTC &= ~((1 << PC4) | (1 << PC5));
	PORTC |= (1 << PC4);

	OCR2 = speed;
}

void inline right_motor_backward(int speed)
{
	PORTC &= ~((1 << PC7) | (1 << PC6));
	PORTC |= (1 << PC6);
	

	OCR0 = speed;
}

void inline left_motor_brake()
{
	PORTC &= ~(1 << PC6);
	PORTC &= ~(1 << PC7);
}

void inline right_motor_brake()
{
	PORTC &= ~(1 << PC4);
	PORTC &= ~(1 << PC5);
}

//--------------------------------------------

void inline robot_forward()
{

	right_motor_forward(255);
	left_motor_forward(255);
}

void inline robot_backward()
{

	right_motor_backward(255);
	left_motor_backward(255);
}

void inline robot_brake()
{
	right_motor_brake();
	left_motor_brake();
}


void inline robot_left()
{

	left_motor_backward(50);
	right_motor_forward(50);
}


void inline robot_right()
{

	left_motor_forward(50);
	right_motor_backward(50);
}


int single_conversion(uint8_t channel)
{
	int rez;
	
	ADMUX &= 0xE0;
	ADMUX |= channel;
	
	ADCSRA |= (1 << ADSC);
	
	while ((ADCSRA & (1<<ADSC)))
		;
	
	rez = ADCL;
	rez += (ADCH << 8);
	
	return rez;
}

int main()
{
	int i;
	
	
	DDRD = 0xBF;
	PORTD = 0x40;

	DDRC = 0xFF;
	PORTC = 0x0;
	
	init_timers();
	init_adc();
	
	PORTC |= (1 << PC0);
	PORTC |= (1 << PC2);
	robot_brake();
	_delay_ms(2000);
	
	while (1)
	{
		for (i = 0; i < 5; i++)
			sensor[i] = single_conversion(chan[i]);
			
		if (sensor[3] > threshold[3] || sensor[4] > threshold[4])
		{
			PORTC |= (1 << PC0);
			PORTC &= ~(1 << PC2);
			
			robot_right();
			goto next;
		}
		if (sensor[0] > threshold[0] || sensor[1] > threshold[1])
		{
			PORTC &= ~(1 << PC0);
			PORTC |= (1 << PC2);
			
			robot_left();
			goto next;
		}
		
		PORTC |= (1 << PC0) | (1 << PC2);
		
		robot_forward();
		
		
		next:
		_delay_ms(70);
	}
}